# -*- coding: utf-8 -*-

"""
video.py est un module permettant de faire un fichier video à partir d'un
tableau de positions-vitesses tel que pysatellite peut le réaliser.
"""

import math, datetime, tempfile, os.path, os
from astres import Astre
from PyQt4.QtGui import *
from threading import *
from debug import Debug
import flottant as flottant

class Cinema(Thread):
    def __init__(self,repertoire, astre, dateorigine, liste_temps, liste_pos, geometrie, pas=1, nettoie=True, boum=-1.0, debugger=Debug(0)):
        Thread.__init__(self)
        self.repertoire=repertoire
        self.astre=astre
        self.dateorigine=dateorigine
        self.liste_temps=liste_temps
        self.liste_pos=liste_pos
        self.geometrie=geometrie
        self.pas=pas
        self.nettoie=nettoie
        self.boum=boum
        self.dir=tempfile.mkdtemp("","pysat")
        self.fini=False
        self.nbImage=0
        self.debug=debugger
        
    def run(self):
        self.video()
        os.system("vlc --loop %s > /dev/null 2>&1" %os.path.join(self.dir,"out.avi"))
        if self.nettoie: os.system("rm -r %s" %self.dir)
        
    def video(self):
        self.images()
        cmd="ffmpeg -r 25 -f image2 -i %s -f avi -vcodec mpeg1video -b 800k %s > /dev/null 2>&1" %(os.path.join(self.dir, "%04d.jpg"), os.path.join(self.dir, "out.avi"))
        os.system(cmd)

    def images(self):
        listenum= range(0, len(self.liste_temps), self.pas)
        for i in listenum:
            if self.fini:
                return
            temps=self.liste_temps[i]
            if self.boum > 0 and temps > self.boum:
                self.imageCrash(self.nbImage)
                self.fini=True
                return
            x=self.liste_pos[i][0]
            y=self.liste_pos[i][1]
            z=self.liste_pos[i][2]
            cmd=self.xplanetCmd(temps, x, y, z,
                                os.path.join(self.dir, "%04d.jpg" %self.nbImage))
            os.system(cmd)
            self.nbImage+=1

    def imageCrash(self,num):
        for n in range(20):
            nomfichier=os.path.join(self.dir, "%04d.jpg" %(num+n))
            (w,h)=self.geometrie.split("x")
            img=QImage(int(w), int(h), QImage.Format_RGB32)
            img.fill(QColor("red").rgb())
            img.save(nomfichier)
    
    def xplanetCmd(self,temps, x, y, z, nomfichier):
        """
        dateorigine est un objet datetime fixe,
        temps est une durée en seconde
        """
        a=Astre(self.astre)
        td=datetime.timedelta(seconds=temps)
        date=self.dateorigine+td
        dateXplanet=date.strftime("%Y%m%d.%H%M%S")

        rayon=1.0*(x**2+y**2+z**2)**0.5
        range=rayon/a.rayon
        if range > 1.0:
            radius=100*math.asin(1.0/range)
        else:
            radius=9000
        longitude0=360.0*temps/(24*3600*flottant.traduit(a.rotationSiderale))
        longitude=180/math.pi*math.atan2(y,x)-90
        latitude=180/math.pi*math.atan2(z,(x**2+y**2)**0.5)
        cmd="xplanet -date %s -radius %s -num_times 1 -output '%s' -geometry %s -origin %s -range %s -longitude %s -latitude %s -starmap BSC -searchdir %s -body %s -label >/dev/null 2>&1" %(dateXplanet, radius, nomfichier, self.geometrie, self.astre, range, int(a.flip)*(longitude-longitude0), latitude, self.repertoire.chemin("textures"), self.astre)
        self.debug(4,u"Lancemende de «%s»" %cmd)
        return cmd
    



#-*- coding: utf-8 -*-

from PyQt4.QtCore import *
from PyQt4.QtGui import *

class Point(QLabel):
    def __init__(self, parent, point, color, numero, app, pred=None,type_de_point="petit"):
        """
        Crée un point graphique. Paramètres :
        parent : widget parent
        point : coordonnées (de type vecteur)
        color : couleur
        numero : numéro à afficher
        app : l'application qui commande
        pred : le point prédecesseur
        type_de_point : un paramètre de style
        """
        QLabel.__init__(self, parent)
        self.app=app
        self.point, self.color = point,color
        #self.setGeometry(QRect(0,0,640,480))
        self.setGeometry(QRect(0,0,parent.width(),parent.height()))
        self.numero=numero
        self.type_de_point = type_de_point
        if type_de_point=="petit" :
            self.largeur=2
        elif type_de_point=="gros" :
            self.largeur=4
        else :
            self.largeur=2

    def icone(self,nom):
        return self.app.rep.fichier("icones",nom)
    
    def paintEvent(self,event):
        self.painter = QPainter()
        self.painter.begin(self)
        self.painter.setPen(QColor(self.color))
        self.painter.translate(self.point[0], self.point[1])
        if self.type_de_point=="boum" :
            self.image_sat=QPixmap(self.icone("sat_mini_boum.png"))
            self.painter.drawPixmap(0,0,self.image_sat)
        elif self.type_de_point=="gros" :
            self.image_sat=QPixmap(self.icone("sat_mini.png"))
            self.painter.drawPixmap(0,0,self.image_sat)
            self.painter.drawLine(-self.largeur,0,self.largeur,0)
            self.painter.drawLine(0,-self.largeur,0,self.largeur)
        elif self.type_de_point=="petit" :
            self.painter.drawLine(-self.largeur,0,self.largeur,0)
            self.painter.drawLine(0,-self.largeur,0,self.largeur)
        
        self.painter.end()



#-*- coding: utf-8 -*-

licence="""
    pysatellites : a program to plot trajectories of satellites
    Copyright (C) 2007-2008 Jean-Baptiste Butet <ashashiwa@gmail.com>,
              (C) 2007-2008 Georges Khaznadar <georgesk@ofset.org>
    

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

licence_fr="""
    pysatellites : un programme pour tracer les trajectoires de satellites
    Copyright (C) 2007-2008 Jean-Baptiste Butet <ashashiwa@gmail.com>,
              (C) 2007-2008 Georges Khaznadar <georgesk@ofset.org>
    
    Ce projet est un logiciel libre : vous pouvez le redistribuer, le modifier selon les terme de la GPL (GNU Public License) dans les termes de la Free Software Foundation concernant la version 3 ou plus de la dite licence.
    
    Ce programme est fait avec l'espoir qu'il sera utile mais SANS AUCUNE GARANTIE. Lisez la licence pour plus de détails.
    
    <http://www.gnu.org/licenses/>.
"""

import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from debug import Debug
from mainWindow import StartQT4

                

        
def usage():
    print ("Usage : pysatellites [-h | --help] [-d n | --debug=n] [-f fichier | --fichier=fichier]")
    sys.exit(0)

def run():
    app = QApplication(sys.argv)

    #translation
    #locale = QLocale.system().name()

    #qtTranslator = QTranslator()
    #if qtTranslator.load("qt_" + locale):
    #    app.installTranslator(qtTranslator)
    #appTranslator = QTranslator()
        
   # if appTranslator.load("lang/pyfocus_" + locale):
        #app.installTranslator(appTranslator)

    from getopt import getopt
    optlist, argv=getopt(sys.argv[1:],"d:f:h",["debug=","fichier=","help"])

    debugger=Debug(0)
    for (cle,val) in optlist:
        if cle=="-d" or cle=="--debug":
            debugger=Debug(int(val))
        if cle=="-h" or cle=="--help":
            usage()
        if cle=="-f" or cle=="--fichier":
            print ("On ne sait pas encore quoi faire de '%s', la fonctionnalité gouvernée par '%s' reste à implémenter." %(val,cle))

    windows = StartQT4(None, debugger=debugger, app=app)
    windows.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-

class Debug:
    def __init__(self, debugLevel):
        self.debugLevel=debugLevel

    def __call__(self,level,msg):
        if self.debugLevel > level:
            print(msg)



# -*- coding: utf-8 -*-

import os.path

class repertoire:
    def __init__(self, chemin):
        self.chemin0=os.path.abspath(chemin)
        if os.path.isfile(self.chemin0):
            self.chemin0=os.path.dirname(self.chemin0)
        
    def chemin(self,choix="defaut"):
        if choix=="defaut":
            return self.chemin0
        if choix=="textures":
            for d in [self.chemin0, "/usr/share/pysatellites"]:
                if os.path.exists(d+'/images/earth-clouds.jpg'):
                    return d
            print ("erreur : pas de répertoire des planètes")
            print ("=== Il manque les textures de Stellarium ===")
            raise(IOError)
        elif os.path.isdir(os.path.join(self.chemin0,choix)):
            return os.path.join(self.chemin0,choix)
        else:
            raise(IOError)
    def fichier(self,*elementsDeChemin):
        f=self.chemin0
        for e in elementsDeChemin:
            f=os.path.join(f,e)
        return f
        



# -*- coding: utf-8 -*-
# Résolution numérique d'un problème à un corps soumis à un
# potentiel newtonien (en 1/r^2)
# d'équa. diff. vec{r}'' = -k/m vec{r}/r^3
# le vecteur de données considéré sera le vecteur (x, y , vx, vy)
# dont la dérivée est (vx, vy, -k/m*x/(x^2+y^2)^3/2, -k/m*y/(x^2+y^2)^3/2)

from numpy import *
from pylab import *
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from point import Point
import os, tempfile, time
from matplotlib_widget import MyMplCanvas
import flottant as flottant
from rectangle_sensible import Rs
from debug import Debug

def rk4(derivs, y0, t):
    """
    C'est le code de rk4 pris dans le module matplotplib.
    
    Liste des paramètres d'entrée

    derivs :
      une fonction qui accepte en entrée un 6-uplet position,vitesse
      et le papramètre temps, et qui renvoie en sortie un 6-uplet de dérivées.
    y0 :
      un 6-uplet représentant la position et la vitesse initiales
    t :
      une liste de dates régulièrement espacées pour lesquelles on veut
      construire les points et vitessesde la trajectoire

    Résultat de la fonction :
      une liste de 6-uplets représentant les positions et vitesses aux
      instants de la liste des dates données.
    """
    Float=0.0
    try: Ny = len(y0)
    except TypeError:
        yout = zeros( (len(t),), float)
    else:
        yout = zeros( (len(t), Ny), float)
    yout[0] = y0
    i = 0
    for i in arange(len(t)-1):
        thist = t[i]
        dt = t[i+1] - thist
        dt2 = dt/2.0
        y0 = yout[i]
        k1 = asarray(derivs(y0, thist))
        k2 = asarray(derivs(y0 + dt2*k1, thist+dt2))
        k3 = asarray(derivs(y0 + dt2*k2, thist+dt2))
        k4 = asarray(derivs(y0 + dt*k3, thist+dt))
        yout[i+1] = y0 + dt/6.0*(k1 + 2*k2 + 2*k3 + k4)
    return yout


def euler(derivs, y0, t):
    """
    C'est le code de la méthode d'Euler, qui est d'ordre 1 et très simple.
    
    Liste des paramètres d'entrée

    derivs :
      une fonction qui accepte en entrée un 6-uplet position,vitesse
      et le papramètre temps, et qui renvoie en sortie un 6-uplet de dérivées.
    y0 :
      un 6-uplet représentant la position et la vitesse initiales
    t :
      une liste de dates régulièrement espacées pour lesquelles on veut
      construire les points et vitessesde la trajectoire

    Résultat de la fonction :
      une liste de 6-uplets représentant les positions et vitesses aux
      instants de la liste des dates données.
    """
    try: Ny = len(y0)
    except TypeError:
        yout = zeros( (len(t),), Float)
    else:
        yout = zeros( (len(t), Ny), Float)
    yout[0] = y0
    i = 0
    for i in arange(len(t)-1):
        thist = t[i]
        dt = t[i+1] - thist
        y0 = yout[i]
        k1 = asarray(derivs(y0, thist))
        yout[i+1] = y0 + dt*k1
    return yout

def nouvChampGrav(x0,m):
    """Cette fonction renvoie une fonction anonyme représentant un champ
    gravitationnel créé par un objet de masse m immobile aux coordonnées
    spécifiées par x0.
    le profil de la fonction résultat est :
    (vecteur position -> vecteur accélération)
    """
    return lambda x: (-6.67e-11*m*(x[0]-x0[0])/((x[0]-x0[0])**2+(x[1]-x0[1])**2+(x[2]-x0[2])**2)**1.5,
                       -6.67e-11*m*(x[1]-x0[1])/((x[0]-x0[0])**2+(x[1]-x0[1])**2+(x[2]-x0[2])**2)**1.5,
                       -6.67e-11*m*(x[2]-x0[2])/((x[0]-x0[0])**2+(x[1]-x0[1])**2+(x[2]-x0[2])**2)**(1.5))
    
class trajectoire(QObject):
    def __init__(self,dt,t,y0, champ):
        """Les paramètres sont :
        dt     : intervalle de temps
        t      : durée totale de la simulation
        y0     : vecteur à 6 composantes (position, vitesse initiales)
        champ  : une fonction donnant l'accélération à partir de la position
        """
        QObject.__init__(self)
        self.dt = dt
        self.t = arange(0,t,self.dt)
        self.y0 = y0
        self.champ=champ
        self.calcul()

    def dessine(self):
        x=[]
        y=[]
        for point in self.pv :
            x.append(point[0])
            y.append(point[1])
        self.x = x
        self.y = y
        
        plot(self.x,self.y) # le dessin est en projection dans le plan x,y
        axis('equal')
        show()
        
    def derivs(self,x,t):
        """calcul de la dérivée du vecteur à 6 composantes position,vitesse
        """
        g = self.champ(x[0:3])       # le champ de gravité
        return (x[3],                # vitesse x
                x[4],                # vitesse y
                x[5],                # vitesse z
                g[0],                # acceleration x
                g[1],                # acceleration y
                g[2])                # acceleration z

    def calcul(self):
        """lance le calcul de la trajectoire à l'aide de l'algorithme de
        Runge-Kutta qui est d'ordre 4 et rapide en même temps.
        le résultat est dans self.pv, qui est une liste contenant
        les 6-uplets position, vitesse.
        """
        self.pv = rk4(self.derivs, self.y0, self.t)


def calculeNorme(nuplet):
    s=0
    for x in nuplet:
        s+=x**2
    return s**0.5

def projection(n):
    if n >=0:
        return lambda nuplet:nuplet[n]
    else:
        return 0.0
    
class Trajectoire(Rs):
    def __init__(self, parent, rayon_astre, mainWin, debugger=Debug(0)):
        Rs.__init__(self, parent, debugger=debugger)
        self.parent = parent
        self.mainWin=mainWin
        self.milieuX=self.width()/2
        self.milieuY=self.height()/2
        self.rep= mainWin.rep
        #self.setFrameShape(QFrame.Box)
        self.setEchelle(6.4e6/25)
        self.points={}
        self.planetes={}
        self.vitesse=[]
        self.date=[]
        self.dt=1
        self.widget_vit_norm=None
        self.widget_vitx=None
        self.widget_vity=None
        self.boum=-1.0
        self.traj=None
        
        
    def setEchelle(self,val, mode="mppx"):
        """ Régle l'échelle, selon le mode choisi.
        mode=mppx : échelle en mètre par pixel
        mode=max : l'échelle sera ajustée pour que le point (0,val) soit dans
                   la fenêtre de trajectoire (à 95% du maximum)
        """
        if mode=="mppx":
            self.echelle=val
            self.debug(9,"Échelle %s px/m (mode direct)" %self.echelle)
        elif mode=="max":
            self.echelle=val/self.milieuY/0.95
            self.debug(9,"%s m pour %s px" %(val,self.milieuY))
            self.debug(9,"Échelle %s px/m (mode max)" %self.echelle)
        else:
            self.debug(0,"Le mode %s ne convient pas pour setEchelle()" %mode)

    def lance(self):
        self.calcul_parametre()
        if self.mainWin.ui.checkBox_efface.isChecked() :
            self.efface()
            self.debug(10,"efface la trajectoire depuis un lancé")
        a=time.time()

        self.traj= trajectoire(self.dt, self.t, self.pos+self.vit, self.gAstre)
        self.debug(8, "%s, %s, %s, %s" %(self.dt, self.t, self.pos+self.vit, self.gAstre))
        self.debug(5,"calcul en %s secondes" %(time.time()-a))
        self.dessine_trajectoire()
        #dessine les vitesses
        self.grapheV()
        
    def grapheV(self):
        if self.widget_vit_norm != None:
            self.widget_vit_norm.hide()
        if self.widget_vitx != None:
            self.widget_vitx.hide()
        if self.widget_vity != None:
            self.widget_vity.hide()
        self.widget_vit_norme = MyMplCanvas(self.mainWin.ui.label_vit_norm,self.vitesse, calculeNorme, self.date, width=3, height=5, dpi=30, cliquable=True, titre="Norme de la vitesse") 
        self.widget_vitx = MyMplCanvas(self.mainWin.ui.label_vitx,self.vitesse, projection(0), self.date, width=3, height=5, dpi=30, cliquable=True, titre="Abscisse de la vitesse")
        self.widget_vity = MyMplCanvas(self.mainWin.ui.label_vity,self.vitesse, projection(1), self.date, width=3, height=5, dpi=30, cliquable=True, titre="Ordonnée de la vitesse")
        self.widget_vit_norme.show()
        self.widget_vitx.show()
        self.widget_vity.show()
        
    
    def calcul_parametre(self):
        self.masse_astre = self.mainWin.getMasseAstre()
        self.distance_astre = self.mainWin.getDistanceAstre()
        self.vitesse_x,self.vitesse_y = self.mainWin.getVitesse()
        
        self.G=6.67259e-11
        #calcul des énergies massiques (cinétique, potentielle, mécanique)
        Ec_massique=0.5*(self.vitesse_x**2+self.vitesse_y**2)
        Ep_massique=-self.masse_astre*self.G/self.distance_astre
        Em_massique=Ec_massique+Ep_massique
        if Em_massique >= 0:
            self.debug(1,"L'énergie mécanique est excessive (%s J/kg), la trajectoire ne se fermera pas" %Em_massique)
            t=QApplication.translate("MainWindow", "Ce n'est pas un satellite", None, QApplication.UnicodeUTF8)
            q=QApplication.translate("MainWindow", "L'énergie mécanique initiale est positive, l'objet lancé échappera à l'attraction de l'astre. Voulez-vous tracer une partie de la trajectoire ?", None, QApplication.UnicodeUTF8)
            ret=QMessageBox.question (self, t, q, 1, 2)
            if ret==1:
                self.t=30*(self.distance_astre**3/self.masse_astre/self.G)**0.5
                self.dt=self.t/100
            else:
                raise(ValueError)
        else:
            # calcul du grand axe a : Em = -k/2a pour une trajectoire elliptique
            # donc a = -k/2Em
            a = - self.masse_astre*self.G/2/Em_massique

            # calcul de la période, en utilisant la troisième loi de Kepler
            # T²=4pi²/MG*a³
            self.t = 2*pi*(a**3/self.masse_astre/self.G)**0.5
            self.dt = self.t/1000
        self.debug(5,"masse astre %s" %self.masse_astre)
        self.gAstre = nouvChampGrav((0,0,0),self.masse_astre)
        self.pos = (0.0, self.distance_astre, 0)
        self.vit = (self.vitesse_x,self.vitesse_y,0.0) 
        
    
    def dessine_trajectoire(self):
        
        self.debug(9,"dessine la trajectoire")
        self.dessine(self.traj.pv)
        # on teste si toute la trajectoire tien bien là.
        r=self.maxDistance(self.traj.pv)
        if not self.surementVisible(r) and not self.boum > 0:
            t=QApplication.translate("MainWindow", "Changement d'échelle", None, QApplication.UnicodeUTF8)
            q=QApplication.translate("MainWindow", "Un dépassement a été détecté. Voulez-vous changer d'échelle ?", None, QApplication.UnicodeUTF8)
            ret=QMessageBox.question (self, t, q, 1, 2)
            if ret==1:
                self.setEchelle(r,"max")
                self.efface()
                self.dessine(self.traj.pv)

    def maxDistance(self,points_vitesses):
        """renvoie la distance max entre le centre et le satellite en
        projection sur le plan xy.
        """
        r=0
        for pv in points_vitesses:
            if fabs(pv[0])>r: r=fabs(pv[0])
            if fabs(pv[1])>r: r=fabs(pv[1])
        return r
            
        
    def surementVisible(self,r):
        """vrai si un cercle de rayon r est visible à coup sûr"""
        return r/self.echelle < self.milieuY
    
    def efface(self):
        for k in self.points.keys():
            objet=self.points[k]
            objet.hide()
            objet.clear()
        self.points = {}
        self.vitesse = []

    def trace_point(self, x, y, couleur, type="petit", tau=None):
        if tau:
            cle=(x,y,tau)
        else:
            cle=(x,y)
        if cle in self.points.keys():
            # efface des points préexistants de même emplacement
            objet=self.points[cle]
            objet.hide()
            objet.clear()
        self.points[cle]=Point(self, (x,y), couleur, "", self.mainWin,type_de_point=type)
        self.points[cle].show()
        

    def dessine(self,points_vitesses):
        self.boum=-1.0
        OKtau=False
        intervalle=self.mainWin.ui.intervale.text()
        self.tau = int(flottant.traduit(intervalle))
        if self.tau > 60: OKtau=True
        
        self.vitesse = []
        self.date=[]
        tau_entier = 0
        for i in range(0,len(points_vitesses),10):
            # on ne trace qu'un point sur 10, soit 100 points
            # sur les 1000 calculés
            pv=points_vitesses[i]
            pix_x=int(pv[0]/self.echelle+self.milieuX)
            pix_y=int(-pv[1]/self.echelle+self.milieuY)
            
            # pv est un hexuplet : 3 coordonnées de position, 3 de vitesse
            self.vitesse.append((pv[3],pv[4]))
            self.date.append(self.dt*i)
            if ((pix_x-self.milieuX)**2+(pix_y-self.milieuY)**2)**(0.5) < int(self.mainWin.getRayonAstre()/self.echelle) :
                self.trace_point(pix_x, pix_y, "red", type="boum")
                self.boum=self.dt*i
                break
            else:
                self.trace_point(pix_x, pix_y, "red")

            if OKtau:
                try :
                    #dessine un point tous les "tau" secondes si défini.
                    tau = i*self.dt/self.tau

                    if int(tau) > int(tau_entier) :
                        self.trace_point(pix_x, pix_y, "blue", type="gros", tau=tau)
                        tau_entier=int(tau)
                except AttributeError:
                        pass
            self.repaint()
        self.update()
        
    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setBrush(Qt.CrossPattern)
        painter.setPen(Qt.green)
	
        rayon=int(self.mainWin.getRayonAstre()/self.echelle)
        painter.drawLine(self.milieuX, 0, self.milieuX, self.milieuY*2)
        painter.drawLine(0,self.milieuY , 2*self.milieuX, self.milieuY)
        img=self.astreImg
        sourcerect=QRect(0,0,512,512)
        self.milieuX=self.size().width()/2
        self.milieuY=self.size().height()/2
        targetrect=QRect(self.milieuX-rayon,self.milieuY-rayon,2*rayon,2*rayon)
        #TODO : il faudrait redessiner le rectangle en fonction de l'échelle
        #TODO : la taille n'est pas bien calculée quand on redimensionne !!!
        painter.drawEllipse(targetrect)
        painter.drawImage(targetrect,img,sourcerect)
        painter.end()

    def dir(self,choix):
        return self.rep.chemin(choix)

    def debug(self,level,msg):
        self.mainWin.debug(level,msg)
        
    def getPlanete(self,nom):
        if nom not in self.planetes.keys():
            handle, imageFile = tempfile.mkstemp(".png")
            os.close(handle)
            cmd="xplanet -latitude 90 -num_times 1 -glare 10 -body %s -radius 50 -searchdir %s -transpng %s -rotate -70" %(nom,self.dir("textures"),imageFile)
            # la rotation de -70° permet au méridiens français de se trouver
            # au centre de l'image. Cette valeur est empirique et dépend
            # probablement de l'implémentation de Xplanet. Il n'est pas
            # évident de jouer avec l'option -north qui seule permet de
            # contrôler totalement l'orientation de l'image.
            self.debug(8,"Création de l'image de planète par \"%s\"" %cmd)
            os.system(cmd)
            self.planetes[nom]=QImage(imageFile)
            os.system("rm -f %s" %imageFile)
        return self.planetes[nom]

    def choisi_astre(self,nom):
        self.astreImg= self.getPlanete(nom)

        
class Satellite(QLabel):
    def __init__(self, parent, coord_sat, image):
        QLabel.__init__(self, parent)
        self.parent = parent
        self.coord_sat=coord_sat
        self.image = image
    def paintEvent(self, event):
        painter = QPainter()
        painter.setBrush(Qt.CrossPattern)
        painter.setPen(Qt.green)
        painter.begin(self)
        x, y = self.coord_sat.x(), self.coord_sat.y()
        self.debug(5,"%s,%s" %(x,y))
        painter.drawEllipse(x,y,30,16)
        painter.drawPixmap(self.coord_sat, self.image)
        painter.end()
        



#-*- coding: utf-8 -*-

"""
code pour la fenêtre principale de pysatellites
"""

licence="""
    the file mainWindow.py is part of the package pysatellites.
    
    Copyright (C) 2007-2008 Jean-Baptiste Butet <ashashiwa@gmail.com>,
              (C) 2007-2008 Georges Khaznadar <georgesk@ofset.org>
    

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys, os
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from glob import glob
from UI_pysat  import Ui_MainWindow
from traj_satellite  import Trajectoire
from math import pi, cos, sin, fabs
from astres import astreNom
from point import Point
from matplotlib_widget import MyMplCanvas
from repertoire import repertoire
from video import Cinema
import flottant as flottant
from debug import Debug

class StartQT4(QMainWindow):
    def __init__(self, parent, rep=None , debugger=Debug(1), app=None):
        QMainWindow.__init__(self)
        QWidget.__init__(self, parent)
        self.debug=debugger
        self.app=app
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        if rep == None:
            self.rep=repertoire(sys.argv[0])
        else:
            self.rep=rep
        self.trajectoire = Trajectoire(self.ui.afficheur, 6400, self, debugger=self.debug)
        self.initAstres()
        self.ui.masse_astre.setText(QApplication.translate("MainWindow", "6x10^24", None, QApplication.UnicodeUTF8))
        self.ui.rayon_astre.setText(QApplication.translate("MainWindow", "6400", None, QApplication.UnicodeUTF8))
        os.chdir(self.rep.chemin("defaut"))
        self.image_sat=QPixmap("icones/sat_mini.gif")
        self.connexions_signaux()
        self.placeDepart()
        self.cinemaThread=None
        self.progress=None


    def getRayonAstre(self):
        return flottant.traduit(self.ui.rayon_astre.text())*1000

    def placeDepart(self):
        """
        place la croix à la position de départ du satellite et figure
        le vecteur vitesse
        """
        self.trajectoire.efface()
        x=0
        y=(flottant.traduit(self.ui.rayon_astre.text())+flottant.traduit(self.ui.altitude_objet.text()))*1000
        self.trajectoire.setEchelle(y,"max")
        vx=flottant.traduit(self.ui.vitesse_tangentielle_objet.text())
        vy=flottant.traduit(self.ui.vitesse_normale_objet.text())

        self.trajectoire.dessine([(x,y,0,vx,vy,0)])
        self.trajectoire.update()
        
    def initAstres(self):
        """
        Peuple le combo avec les noms d'astres
        """
        for a in astreNom:
            self.ui.astreCombo.addItem(QApplication.translate("MainWindow", a[0], None, QApplication.UnicodeUTF8))
        self.ui.astreCombo.setEditable(False)
        self.ui.astreCombo.setCurrentIndex(0)
        self.astreCourant="earth"
        self.choisi_astre(0)
        
    def connexions_signaux(self):
        QObject.connect(self.ui.Bouton_Lancer,SIGNAL("clicked()"), self.trajectoire.lance)
        QObject.connect(self.ui.Button_efface,SIGNAL("clicked()"), self.efface_trajectoire)
        QObject.connect(self.ui.bouton_video,SIGNAL("clicked()"), self.cinema)
        QObject.connect(self.ui.altitude_objet,SIGNAL("editingFinished()"), self.placeDepart)
        QObject.connect(self.ui.astreCombo,SIGNAL("currentIndexChanged(int)"),self.choisi_astre)
        QObject.connect(self.ui.radioButton_Frenet,SIGNAL("toggled(bool)"), self.choisi_coordoonees)
        #QObject.connect(self.ui.checkBox_efface,SIGNAL("stateChanged(bool)"), self.change_comportement_effacege)
        timer = QTimer(self);
        QObject.connect(timer, SIGNAL("timeout()"), self.routines);
        timer.start(1000);


        self.ui.mTerre.setReadOnly (True)
        self.ui.rTerre.setReadOnly (True)
        self.ui.auSujetAstre.setReadOnly (True)
    
    def change_comportement_effacege(self,int):
        pass
    
    def efface_trajectoire(self):
        self.trajectoire.efface()
        self.placeDepart()

    def cinema(self):
        if self.cinemaThread!=None and self.cinemaThread.isAlive():
            return
        import datetime,copy
        from numpy import arange
        date=datetime.datetime(2008,4,25)
        date=date.today()
        if self.trajectoire.traj != None:
            liste_temps=arange(0,self.trajectoire.t,self.trajectoire.dt)
            pas=10 # une image pour 10 calculs numériques
            titre=QApplication.translate("MainWindow", "Calcul de la vidéo", None, QApplication.UnicodeUTF8)
            legende=QApplication.translate("MainWindow", "Avancement ...", None, QApplication.UnicodeUTF8)
            echap=QApplication.translate("MainWindow", "Arrêt", None, QApplication.UnicodeUTF8)
            self.progress=QProgressDialog(legende,echap,0,len(liste_temps)/pas)
            self.progress.setWindowTitle(titre)
            self.progress.setValue(0)
            self.progress.show()

            # on lance le thread avec une copie de la liste calculée
            self.cinemaThread=Cinema(self.rep, self.astreCourant, date,
                                     liste_temps,
                                     copy.copy(self.trajectoire.traj.pv),
                                     "380x240",
                                     pas=pas, boum=self.trajectoire.boum,
                                     debugger=self.debug)
            self.cinemaThread.start()
            
    def routines(self):
        if self.cinemaThread!=None and self.cinemaThread.isAlive():
            self.progress.setValue(self.cinemaThread.nbImage)
            if self.progress.wasCanceled():
                self.progress.close()
                self.cinemaThread.fini=True
            if self.cinemaThread.fini:
                self.progress.setValue(self.progress.maximum()+1)
                if self.progress: self.progress.close()
            
    
    def getMasseAstre(self):
        return flottant.traduit(self.ui.masse_astre.text())
    def getDistanceAstre(self):
        return (flottant.traduit(self.ui.rayon_astre.text()) + flottant.traduit(self.ui.altitude_objet.text()))*1000 #passe en mètres
    def getVitesse(self):
        return (flottant.traduit(self.ui.vitesse_tangentielle_objet.text()),
                flottant.traduit(self.ui.vitesse_normale_objet.text()))
                
        
    def choisi_coordoonees(self,bool):
        if self.ui.radioButton_Cartesiennes.isChecked()==True :
            self.ui.label_V1.setText(QApplication.translate("MainWindow", "Vitesse selon Ox", None, QApplication.UnicodeUTF8))
            self.ui.label_V2.setText(QApplication.translate("MainWindow", "Vitesse selon Oy", None, QApplication.UnicodeUTF8))
            
        elif self.ui.radioButton_Cartesiennes.isChecked()==False :
            self.ui.label_V1.setText(QApplication.translate("MainWindow", "Vitesse Tangentielle", None, QApplication.UnicodeUTF8))
            self.ui.label_V2.setText(QApplication.translate("MainWindow", "Vitesse Normale", None, QApplication.UnicodeUTF8))


    def choisi_astre(self,int):
        """
        choisit un astre parmi la liste disponible, sur la base du texte
        couramment sélectionné dans le combo
        @param int non utiliséé
        """
        mt0=astreNom[0][2]
        rt0=astreNom[0][3]
        astre=self.ui.astreCombo.currentText()
        for a in astreNom:
            if a[0]==astre:
                self.trajectoire.choisi_astre(a[1])
                self.astreCourant=a[1]
                masse_astre=a[2]
                rayon_astre=a[3]
                self.ui.masse_astre.setText(QApplication.translate("MainWindow", masse_astre, None, QApplication.UnicodeUTF8))
                self.ui.rayon_astre.setText(QApplication.translate("MainWindow", rayon_astre, None, QApplication.UnicodeUTF8))
                mt=flottant.traduit(masse_astre)/flottant.traduit(mt0)
                rt=flottant.traduit(rayon_astre)/flottant.traduit(rt0)
                mt="%5g" %mt
                rt="%5g" %rt
                self.ui.mTerre.setText(QApplication.translate("MainWindow", mt, None, QApplication.UnicodeUTF8))
                self.ui.rTerre.setText(QApplication.translate("MainWindow", rt, None, QApplication.UnicodeUTF8))
                self.ui.auSujetAstre.setText(QApplication.translate("MainWindow", a[4], None, QApplication.UnicodeUTF8))
                self.placeDepart()
                self.trajectoire.update()
                return
        self.debug(0,"Astre inconnu : %s" %astre)



# -*- coding: utf-8 -*-

def traduit(chaine):
    """cette méthode vérifie la validité de la chaîne en fonction de sa provenance et, au besoin, transforme des expressions possibles (10^11) en grandeur acceptée par python"""
    # on force le type chaîne pour pouvoir faire des évaluations.
    chaine=str(chaine).replace(" ","") # et retrait de tous les espaces

    #vérification de la présence d'un float correct, sinon tente des modifs.
    try : 
        chaine=float(eval(chaine))
    except : 
        chaine=chaine.replace("10^","e")
        #remplace la chaine 10^ par e
    else:
        return chaine

    try : 
        chaine=float(eval(chaine))
    except : 
        #remplace les "x" pour la multiplication
        chaine=chaine.replace("x","*")
        chaine=chaine.replace("X","*")
    else:
        return chaine

    try : 
        chaine=float(eval(chaine))
    except : 
        #remplace les "*" devant un "e"
        chaine=chaine.replace("*e","e")
    else:
        return chaine

    try : 
        chaine=float(eval(chaine))
    except : 
        #self.debug(0,u"Erreur : %s, même après les tranformations, n'est pas une expression acceptable" %chaine)
        return 1.0
    else:
        return chaine
    return 1.0




# -*- coding: utf-8 -*-

from PyQt4.QtGui import *
from PyQt4.QtCore import *
from debug import Debug

class Rs(QWidget):
    def __init__(self, parent, geometry=None,
                 image=None, text=None,
                 color=QColor("grey"),
                 debuglevel=0,
                 onPress=None,
                 onRelease=None,
                 onDoubleClick=None,
                 onMove=None,
                 debugger=Debug(0)):
        QWidget.__init__(self,parent)
        if geometry==None: # le widget recouvrira le parent
            if parent!=None:
                self.setGeometry(QRect(0,0,parent.width(),parent.height()))
            else:
                self.setGeometry(QRect(0,0,100,100))
        else:
            self.setGeometry(geometry)
        self.setMouseTracking(True)
        self.image=image
        self.text=text
        self.color=color
        self.debug=debugger
        self.onPress=onPress
        self.onRelease=onRelease
        self.onDoubleClick=onDoubleClick
        self.onMove=onMove
        
    def mousePressEvent(self,ev):
        if self.onPress != None:
            return self.onPress(ev)
        else:
            self.debug(9, "event onPress still to implement")
    def mouseReleaseEvent(self,ev):
        if self.onRelease != None:
            return self.onRelease(ev)
        else:
            self.debug(9, "event onRelease still to implement")
    def mouseMoveEvent(self,ev):
        if self.onMove != None:
            return self.onMove(ev)
        else:
            self.debug(9, "event onMove still to implement ... x=%s, y=%s" %(ev.x(),ev.y()))
    def mouseDoubleClickEvent(self,ev):
        if self.onDoubleClick != None:
            return self.onDoubleClick(ev)
        else:
            self.debug(9, "event onDoubleClick still to implement")
        
    def paintEvent(self, event):
        QWidget.paintEvent(self,event)
        self.painter = QPainter()
        self.painter.begin(self)
        if self.color !=None:
            self.painter.fillRect(QRect(0,0,self.width(),self.height()),
                                  self.color)
        if self.image != None:
            self.painter.drawImage(0,0,self.image)
        if self.text != None:
            self.painter.drawText(0,0,self.text)
        self.painter.end()

class RsImage(Rs):
    """Une classe dérivée du rectangle sensible Rs, qui contient
    une image et se place en x,y sur le parent
    """
    def __init__(self, parent, x, y, image,
                 text=None,
                 color=QColor("grey"),
                 onPress=None,
                 onRelease=None,
                 onDoubleClick=None,
                 onMove=None,
                 debugger=Debug(0)):
        rect=QRect(x,y,image.size().width(), image.size().height())
        Rs.__init__(self, parent, rect, image=image,
                    text=None,
                    color=color,
                    onPress=onPress,
                    onRelease=onRelease,
                    onDoubleClick=onDoubleClick,
                    onMove=onMove,
                    debugger=debugger)



# embedding_in_qt4.py --- Simple Qt4 application embedding matplotlib canvases
#
# Copyright (C) 2005 Florent Rougon
#               2006 Darren Dale
#
# This file is an example program for matplotlib. It may be used and
# modified with no restriction; raw copies as well as modified versions
# may be distributed without limitation.

import sys, os, random
from PyQt4 import QtGui, QtCore
from numpy import arange, sin, pi
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from UI_graphe import Ui_Graphe

progname = os.path.basename(sys.argv[0])
progversion = "0.1"


class MyMplCanvas(FigureCanvas):
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
    def __init__(self,  parent, donnees, traitement, dates, width=0, height=0, dpi=100, cliquable=False, titre=""):
        self.fig = Figure(figsize=(width, height), dpi=dpi )
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.axes = self.fig.add_subplot(111)
        a = self.axes.set_axis_off()
        # We want the axes cleared every time plot() is called
        self.axes.axison = False
        self.axes.axis('off')
        self.axes.hold(False)

        self.cliquable=cliquable
        self.donnees=donnees
        self.traitement=traitement
        self.dates=dates
        self.titre=titre

        self.plot(donnees, traitement, dates)


    def mouseReleaseEvent(self, event):
        if self.cliquable:
            self.fils=QDialog()
            self.fils.ui=Ui_Graphe()
            self.fils.ui.setupUi(self.fils)
            self.fils.setWindowTitle(QtGui.QApplication.translate("Graphe", self.titre, None, QtGui.QApplication.UnicodeUTF8))
            self.fils.show()
            ratio=2
            w=3*ratio
            h=5*ratio
            d=90/ratio
            self.fils.canvas=MyMplCanvas(self.fils.ui.grapheLabel,self.donnees, self.traitement, self.dates, width=w, height=h, dpi=d, cliquable=False, titre=self.titre)
            self.fils.canvas.show()
        
    def sizeHint(self):
        w, h = self.get_width_height()
        #print "w, h", w, h
        return QtCore.QSize(w, h)

    def minimumSizeHint(self):
        return QtCore.QSize(10, 10)
    
    def plot(self,donnees, traitement, dates):
        d=[]
        for dd in donnees:
            d.append(traitement(dd))
        self.axes.plot(dates,d)

